{ ----------------------------------------------------------------------------
  Tips.pas                 Copyright  1998-99 by DithoSoft Software Solutions
  Version 1.00b
  ----------------------------------------------------------------------------
  The TTips component provides an easy way to display a "Tip of the day"
  dialog as it is known from many applications.
  One advantage of TTips is that no bulky .DFM file is carried along! The Tips
  dialog is created "manually" at runtime.
  The strings for the tips can either be added at design time or can be stored
  in the Registry.
  ----------------------------------------------------------------------------
  Instructions:
    -  In case you need to localize the captions of the dialog components,
       please modify the "TipCaptions.rc" resource script and compile it using
       BRCC32.EXE to create the "TIPCAPTIONS.RES" resource file.
    -  If you need to change the resource IDs of the strings, please remeber
       to change the constants in the unit as well!
  ----------------------------------------------------------------------------
  Registry format:
  Add one string value for every tip. Rename the value to a number and enter
  the tip.
  For example:  Value "1", Contents "Tip1"
                Value "2", Contents "Tip2"
  ----------------------------------------------------------------------------
  Properties
    property CurrentTip: Integer
             -> Get or set the index of the current tip (-1 = no tip)

    property Glyph: TIcon
             -> Get or set the user defined tip icon

    property GlyphStyle: TGlyphStyle
             -> Get or set the style of the icon

    property RegParentKey: String
             -> Get or set the parent key of the key containing the tips

    property RegSubKey: String
             -> Get or set the key containing the tips

    property ShowOnStartup: Boolean
             -> Get or set the state of the "Show on StartUp" checkbox

    property Tips: TStringList read FTips write SetTips;
             -> List of all tips

    property UseRegistry: Boolean
             -> Indicates whether the tips should be loaded from the registry

  Events
    property OnNextTip: TNotifyEvent
             -> Fired when the "Next" button is clicked

    property OnCloseDialog: TNotifyEvent
             -> Fired when the "Close" button is clicked
  ----------------------------------------------------------------------------
  Version 1.00
          Initial release.
  ----------------------------------------------------------------------------
  homepage: http://dithosoft.cjb.net
  E-Mail:   thorsten.dittmar@debitel.net
  ---------------------------------------------------------------------------- }
unit Tips;
{$R TIPSGLYPHS.RES}
{$R TIPSCAPTIONS.RES}

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, StdCtrls, ExtCtrls,
  Forms, Dialogs;

type
  { TGlyphStyle }
  TGlyphStyle = (gsLightBulb, gsClown, gsHelp, gsArrow, gsTorch, gsCustom);

  { TTips class }
  TTipDialog = class;
  TTips = class(TComponent)
  private
    FCurrentTip     : Integer;
    FGlyph          : TIcon;
    FGlyphStyle     : TGlyphStyle;
    FRegParentKey,
    FRegSubKey      : String;
    FShowOnStartup  : Boolean;
    FTips           : TStringList;
    FUseRegistry    : Boolean;
    FDialog         : TTipDialog;
    FOnNextTip,
    FOnCloseDialog  : TNotifyEvent;
    procedure SetCurrentTip(Value: Integer);
    procedure SetGlyph(Value: TIcon);
    procedure SetGlyphStyle(Value: TGlyphStyle);
    procedure SetRegParentKey(Value: String);
    procedure SetRegSubKey(Value: String);
    procedure SetShowOnStartUp(Value: Boolean);
    procedure SetTips(Value: TStringList);
    procedure SetUseRegistry(Value: Boolean);
    procedure DialogPaintBoxPaint(Sender: TObject);
    procedure DialogCheckBoxClick(Sender: TObject);
    procedure DialogNextButtonClick(Sender: TObject);
    procedure DialogCloseButtonClick(Sender: TObject);
  protected
    procedure Loaded; override;
    procedure LoadRegTips; virtual;
    procedure DoShowTip; virtual;
  public
    constructor Create(AOwner: TComponent); override;
    destructor  Destroy; override;
    procedure   ShowTip; dynamic;
  published
    property CurrentTip: Integer read FCurrentTip write SetCurrentTip;
    property Glyph: TIcon read FGlyph write SetGlyph;
    property GlyphStyle: TGlyphStyle read FGlyphStyle write SetGlyphStyle;
    property RegParentKey: String read FRegParentKey write SetRegParentKey;
    property RegSubKey: String read FRegSubKey write SetRegSubKey;
    property ShowOnStartup: Boolean read FShowOnStartup write SetShowOnStartup;
    property Tips: TStringList read FTips write SetTips;
    property UseRegistry: Boolean read FUseRegistry write SetUseRegistry;
    property OnNextTip: TNotifyEvent read FOnNextTip write FOnNextTip;
    property OnCloseDialog: TNotifyEvent read FOnCloseDialog write FOnCloseDialog;
  end;

  TTipDialog = class(TForm)
  private
    PaintBox : TPaintBox;
    TipLabel : TLabel;
  end;

procedure Register;

implementation

uses
  Registry;

const
  { Resource IDs of the strings }
  STipOfTheDay        : Integer = 32768;
  SDidYouKnow         : Integer = 32769;
  SShowTipsOnStartup  : Integer = 32770;
  SNextTip            : Integer = 32771;
  SClose              : Integer = 32772;

  { Array of glyph resource names }
  GlyphNames  : Array[TGlyphStyle] of String =
                ('TIPICON_BULB','TIPICON_CLOWN','TIPICON_QUESTION',
                 'TIPICON_ARROW','TIPICON_TORCH','');

procedure Register;
begin
  RegisterComponents('Freeware', [TTips]);
end;

{ TTips }
constructor TTips.Create(AOwner: TComponent);
begin
  inherited Create(AOwner);

  { Initialize }
  FCurrentTip    := -1;
  FGlyph         := TIcon.Create;
  FGlyphStyle    := gsLightBulb;
  FRegParentKey  := '';
  FRegSubKey     := '';
  FShowOnStartup := True;
  FTips          := TStringList.Create;
  FUseRegistry   := False;
  FDialog        := nil;
end;

destructor TTips.Destroy;
begin
  FGlyph.Free;
  FTips.Free;
  inherited Destroy;
end;

procedure TTips.Loaded;
begin
  inherited Loaded;

  { If not in Design mode, load the strings from the registry if necessary }
  if not (csDesigning in ComponentState) then
    LoadRegTips;
end;

procedure TTips.LoadRegTips;
var
  RegIni  : TRegIniFile;
  TipList : TStringList;
  i       : Integer;
begin
  { Load the tips from the registry }
  if not FUseRegistry then exit;

  if (Trim(FRegParentKey) <> '') and (Trim(FRegSubKey) <> '') then begin
    RegIni := TRegIniFile.Create(FRegParentKey);
    TipList := TStringList.Create;
    try
      RegIni.ReadSectionValues(FRegSubKey,TipList);
      for i := 0 to Pred(TipList.Count) do
        FTips.Add(TipList.Values[TipList.Names[i]]);
    finally
      RegIni.Free;
      TipList.Free;
    end;
  end;
end;

procedure TTips.ShowTip;
begin
  { Show the tip dialog by calling DoShowTip }
  DoShowTip;
end;

procedure TTips.DoShowTip;
var
  Button: TButton;
begin
  if Assigned(FDialog) then
    FDialog.BringToFront
  else begin
    { If no other dialog was found, create a new one and show it }
    FDialog := TTipDialog.CreateNew(Application);
    try
      with FDialog do begin
        Name         := 'TDailyTip';
        Caption      := LoadStr(STipOfTheDay);
        BorderIcons  := [];
        BorderStyle  := bsDialog;
        FormStyle    := fsStayOnTop;
        ClientWidth  := 337;
        ClientHeight := 231;
        Top          := (Screen.Height-Height) div 2;
        Left         := (Screen.Width-Width) div 2;

        { Insert the bevel }
        with TBevel.Create(FDialog) do begin
          Parent     := FDialog;
          Name       := 'Bevel';
          Shape      := bsFrame;
          SetBounds(8,8,321,185);
        end;

        { Insert the paintbox }
        PaintBox := TPaintBox.Create(FDialog);
        with PaintBox do begin
          Parent     := FDialog;
          Name       := 'PaintBox';
          Color      := clWhite;
          SetBounds(16,16,305,169);
          OnPaint    := DialogPaintBoxPaint;
        end;

        { Insert the label }
        TipLabel := TLabel.Create(FDialog);
        with TipLabel do begin
          Parent        := FDialog;
          AutoSize      := False;
          Color         := clWhite;
          Name          := 'TipLabel';
          ShowAccelChar := False;
          WordWrap      := True;
          SetBounds(24,72,289,105);
          if CurrentTip > -1 then
            Caption       := FTips[CurrentTip]
          else
            Caption       := '';
        end;

        { Insert checkbox }
        with TCheckBox.Create(FDialog) do begin
          Parent        := FDialog;
          Caption       := LoadStr(SShowTipsOnStartup);
          Checked       := FShowOnStartup;
          Name          := 'CheckBox';
          OnClick       := DialogCheckBoxClick;
          SetBounds(8,203,129,17);
        end;

        { Insert "Next" button }
        Button := TButton.Create(FDialog);
        with Button do begin
          Parent        := FDialog;
          Caption       := LoadStr(SNextTip);
          Name          := 'NextButton';
          OnClick       := DialogNextButtonClick;
          SetBounds(160,200,81,23);
        end;

        { Insert "Close" button }
        with TButton.Create(FDialog) do begin
          Parent        := FDialog;
          Caption       := LoadStr(SClose);
          Name          := 'CloseButton';
          OnClick       := DialogCloseButtonClick;
          SetBounds(248,200,81,23);
        end;
        ActiveControl := Button;
      end;
      FDialog.Show;
    except
      FDialog.Free;
    end;
  end;
end;

procedure TTips.SetCurrentTip(Value: Integer);
begin
  if FCurrentTip <> Value then begin
    FCurrentTip := Value;
    if not (csLoading in ComponentState) and (FCurrentTip > Pred(FTips.Count)) then
      FCurrentTip := 0;

    if Assigned(FDialog) then
      if FCurrentTip > -1 then
        FDialog.TipLabel.Caption := FTips[FCurrentTip]
      else
        FDialog.TipLabel.Caption := '';
  end;
end;

procedure TTips.SetGlyph(Value: TIcon);
begin
  if Assigned(Value) then begin
    FGlyph.Assign(Value);
    GlyphStyle := gsCustom;
  end;
end;

procedure TTips.SetGlyphStyle(Value: TGlyphStyle);
begin
  if FGlyphStyle <> Value then
    FGlyphStyle := Value;
end;

procedure TTips.SetRegParentKey(Value: String);
begin
  if FRegParentKey <> Value then
    FRegParentKey := Value;
end;

procedure TTips.SetRegSubKey(Value: String);
begin
  if FRegSubKey <> Value then
    FRegSubKey := Value;
end;

procedure TTips.SetShowOnStartup(Value: Boolean);
begin
  if FShowOnStartup <> Value then
    FShowOnStartup := Value;
end;

procedure TTips.SetTips(Value: TStringList);
begin
  if Assigned(Value) then begin
    FTips.Assign(Value);
    if FCurrentTip > Pred(FTips.Count) then
      FCurrentTip := Pred(FTips.Count);
  end;
end;

procedure TTips.SetUseRegistry(Value: Boolean);
begin
  if FUseRegistry <> Value then
    FUseRegistry := Value;
end;

procedure TTips.DialogPaintBoxPaint(Sender: TObject);
var
  MemBit    : TBitmap;
  GlyphIcon : TIcon;
begin
  { Paint the icon and the "Did you know..." }
  with FDialog do begin
    MemBit := TBitmap.Create;
    try
      { Adjust colors first }
      MemBit.Canvas.Brush.Color := clWhite;
      MemBit.Canvas.Pen.Color   := clBlack;

      { Adjust with and height then and paint the border }
      MemBit.Width              := PaintBox.ClientWidth;
      MemBit.Height             := PaintBox.ClientHeight;
      MemBit.Canvas.MoveTo(0,0);
      MemBit.Canvas.LineTo(MemBit.Width-1,0);
      MemBit.Canvas.LineTo(MemBit.Width-1,MemBit.Height-1);
      MemBit.Canvas.LineTo(0,MemBit.Height-1);
      MemBit.Canvas.LineTo(0,0);

      { Paint the icon }
      GlyphIcon := TIcon.Create;
      try
        if GlyphStyle = gsCustom then
          GlyphIcon.Assign(Glyph)
        else begin
          GlyphIcon.ReleaseHandle;
          GlyphIcon.Handle := LoadIcon(HInstance,PChar(GlyphNames[GlyphStyle]));
        end;
        DrawIcon(MemBit.Canvas.Handle,8,8,GlyphIcon.Handle);
      finally
        GlyphIcon.Free;
      end;

      { Paint the "Did you know..." }
      MemBit.Canvas.Font.Handle := GetStockObject(SYSTEM_FONT);
      MemBit.Canvas.TextOut(48,16,LoadStr(SDidYouKnow));

      { Draw the memory bitmap to the paintbox canvas }
      BitBlt(PaintBox.Canvas.Handle,0,0,PaintBox.ClientWidth,PaintBox.ClientHeight,
             MemBit.Canvas.Handle,0,0,SRCCOPY);
    finally
      MemBit.Free;
    end;
  end;
end;

procedure TTips.DialogCheckBoxClick(Sender: TObject);
begin
  { Set property to the state of the check box }
  ShowOnStartUp := (Sender as TCheckBox).Checked;
end;

procedure TTips.DialogNextButtonClick(Sender: TObject);
begin
  { Increase FCurrentTip }
  CurrentTip := Succ(CurrentTip);
  if Assigned(FOnNextTip) then
    FOnNextTip(Self);
end;

procedure TTips.DialogCloseButtonClick(Sender: TObject);
begin
  { Close }
  if Assigned(FOnCloseDialog) then FOnCloseDialog(Self);
  if Assigned(FDialog) then FDialog.Close;
  if Assigned(FDialog) then FDialog.Release;
  FDialog := nil;
end;

end.
